unit ldb;

interface
{---------------------------------------------------------------------------
  This code written by Bill Beam for Delphi 4 after a medium amount of
  research. Questions about this code go to wilbeam@erols.com (see about box).
  For this to work as is you must have msldbusr.dll installed in your windows
  directory.

  D4 conversion was simple.
  Safearray functions now in ACTIVEX unit instead of OLE2
  and of course Anchors and Contraints.

  Completed first D4 cut on 1/25/98
  Modifcations:
  1. Included MSLDBUSR.DLL error messages in this unit 2/3/99.
  2. Added accelerators and made tabbing friendly 2/3/99.
  3. Added file history to menu. Moves the last used file
     to the top of the list. 2/4/99.
  4. Added editbox onchange event. 2/5/99.
  5. Added anchors and contraints to applicable components. 2/5/99.
  6. Added try/except to check error message out of bounds.
     D4 does bounds checking.
----------------------------------------------------------------------------}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, activex, ComCtrls, Menus, Buttons, ImgList;

type
  Tmainform = class(TForm)
    Panel1: TPanel;
    MainMenu1: TMainMenu;
    Files: TMenuItem;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    Open1: TMenuItem;
    Close1: TMenuItem;
    GroupBox1: TGroupBox;
    ListView1: TListView;
    Panel3: TPanel;
    selbtn: TBitBtn;
    Edit1: TEdit;
    Label1: TLabel;
    Panel4: TPanel;
    but1: TBitBtn;
    but3: TBitBtn;
    but4: TBitBtn;
    but2: TBitBtn;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Help1: TMenuItem;
    About1: TMenuItem;
    BitBtn1: TBitBtn;
    ImageList1: TImageList;
    OpenDialog1: TOpenDialog;
    BitBtn2: TBitBtn;
    procedure DoMain;
    function  DLLexists : Boolean;
    procedure UnLoadDLL;
    procedure DestroySafeArray;
    procedure selbtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RB1Click(Sender: TObject);
    procedure Refresh1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Additems( Fnum, Fusr : String );
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Turnbuttonsoff;
    procedure AddHistory;
    procedure HistoryItemClick( Sender :TObject );
    procedure ClearHistory1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Histmaint( dbparm : String );
    procedure Edit1Change(Sender: TObject);
    function  LoadDLLandCreateSafe : Boolean;
private
    { Private declarations }
    fcstr          : PChar;
    dllhandle      : Integer;
    DLLRunProcName : String;
    dllldbgeterror : String;
    dbname         : String;
    OPt            : Integer;
    PSa            : PSafeArray;
    {SAB is bounding info for each dimension.}
    SAB            : Array[1..1] of TsafeArraybound;
    pvdata         : PChar;
    ivect          : Array[ 1..1 ] of Integer;
    sWinDir        : String;
    inidir         : String;
    dlldir         : String;
    iLength        : Integer;
    sHresult       : Hresult;
    jbitmapoff     : Tbitmap;
    Locofsep       : Integer;
    gdatabase      : String;
    HistStr        : TStringlist;
    buttonson      : Boolean;

  public
    { Public declarations }
  end;

type
TDLLRunProc =
  function( PSa : PSafeArray; dbname: String; Opt : Integer ): Integer; stdcall;

var
  mainform   : Tmainform;
  DLLRunProc : TDLLRunProc;
  
const
SHOWDIMS       = TRUE; 

DLLName        : String = 'msldbusr.dll';     //MS dll name
DLLUsers       : String = 'LDBUser_GetUsers'; //function name
DLLErr         : String = 'LDBUser_GetError'; //function name

ininame        : String = 'vaccuser.ini';
DLLmsgs        : Array [1..15] of String =
('Can''t open the LDB file', 'No user connected', 'Can''t create an Array',
 'Can''t redimension Array', 'Invalid argument passed', 'Memory allocation error',
 'Bad index', 'Out of memory', 'Invalid Argument',
 'LDB is suspected as corrupted', 'Invalid argument', 'Unable to read MDB file',
 'Can''t open the MDB file', 'Can''t find the LDB file - Database not in use',
 'Unknown Error');

implementation
{$R *.DFM}
uses aboutp;

//-------------------------------------------------------------------------
procedure Tmainform.DoMain;
var
 numusers:longint;
 i:Integer;
begin
if fileexists(edit1.text) then
   DBName := edit1.text
else
   begin
      if length(edit1.text) <> 0 then
         showmessage('File ' + edit1.text + ' does not exist.')
      else
         showmessage(edit1.text + ' No Database File specified.');
      edit1.setfocus;
      turnbuttonsoff;
      exit;
   end;
histmaint(edit1.text);
AddHistory;
listview1.items.clear;
{note at(@) operator on PSa - opt is set by the button tag }
numusers := DLLRunProc(@PSa,dbname,opt);
if (numusers < 0) then
    try
       additems('Err=' + inttostr(numusers),dllmsgs[abs(numusers)])
    except
       additems('Err=' + inttostr(numusers),dllmsgs[15]); //unknown error
    end
else
   begin
      label1.caption := 'Snap Shot at ' + datetimetostr(now);
      if (numusers > 0) and (opt <> 8) then
         for i := 1 to numusers do
             begin
                  ivect[1] := i;
                  SafeArrayGetElement(PSa,ivect[1],pvData);
                  additems(inttostr(i),pvData);
             end
      else
         if numusers >= 0 then
            additems(inttostr(numusers),' Users');
   end;
end;
//-------------------------------------------------------------------------
Function Tmainform.LoadDLLandCreateSafe : Boolean;
{var
   Ndim           : Integer;
   Uboun          : Integer;
   Lboun          : Integer;}
begin
result := false;
DLLRunProcName := DLLUsers;
DLLLdbGetError := DLLErr;
if not DLLexists then
   exit;
FCStr := PChar(AllocMem(255));
StrPCopy(FCStr,dlldir);
DLLHandle := LoadLibrary(FCStr); {LOAD MsLdbusr.dll}
if DLLHandle = -1 then
   begin
       showmessage('Could not obtain DLL handle');
       freemem(fcstr);
       exit;
   end;
{GET FUNCTION ADDRESS OF LDBUser_GetUsers}
@DLLRunProc := GetProcAddress(DLLHandle,PChar(DLLRunProcName));
if @DLLRunProc = nil then
   begin
        showmessage('Can''t locate function ' + dllusers);
        UnLoadDLL;
        exit;
   end;
{SAFEArray Element count can be anything.  MsLdbusr.dll will
REDIMENSION it based on the actual number of users.  Use the returned
user count to determine the number of elements.  Lower bound must be set
to 1.  Left in (commented out) code to display the shape of the safeArray }
SAB[1].LLbound   := 1;   {lower bound}
SAB[1].celements := 1;   {numer of elements}
PSa := SafeArrayCreate(VT_BSTR,1,SAB); {Request to oleaut32.dll}
if PSa = nil then
   begin
      showmessage('Can''t create safeArray');
      UnloadDLL;
      exit;
   end;
{For debug..check the shape of the redimensioned safeArray by using the
 following example - Displays Lower and Upper bound for each dimension .}

{Ndim := SafeArrayGetDim(PSa); // get the number of dimensions.
while Ndim <> 0 do
    begin
         sHresult := SafeArrayGetUBound(PSa,Ndim,UBoun);  // get UPPER bound.
         sHresult := SafeArrayGetLBound(PSa,Ndim,LBoun);  // get LOWER bound.
         additems('Dim=' + format('%d',[ndim]),
                    format('Low-Bound[%d]',[Lboun]) +
                    format(' Hi-Bound[%d]',[Uboun]));
         dec(Ndim);
    end;}
result := true;
end;
//-------------------------------------------------------------------------
procedure tmainform.UnLoadDLL;
begin
while FreeLibrary(DLLHandle) do
      Application.ProcessMessages;
freemem(fcstr);
end;
//-------------------------------------------------------------------------
procedure tmainform.DestroySafeArray;
begin
sHresult := SafeArrayDestroy(PSa);
if sHresult <> 0 then
   showmessage('Could''t destroy SafeArray');
end;
//-------------------------------------------------------------------------
function Tmainform.DLLexists : Boolean;
begin
result := fileexists(dllDir);
if not result then
   showmessage(dlldir + ' does not exist.');
end;
//-------------------------------------------------------------------------
procedure Tmainform.selbtnClick(Sender: TObject);
begin
with opendialog1 do
     if execute then
        begin
            gdatabase := filename;      //set database file name
            edit1.text := gdatabase;
        end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.Histmaint(dbparm:String);
var
  tempi : Integer;
begin
with HistStr do
     begin
         tempi := indexof(dbparm);     //see if already in list
         if tempi < 0 then
            begin                      //its a new entry
               insert(0,dbparm);       //insert new entry at the top
               if count = 11 then
                  delete(10);          //only maintain 10 entries 0-9
            end
         else
            begin                      //already on the list
              delete(tempi);           //delete the item
              Insert(0,dbparm);        //move it to the top
            end;
     end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.FormCreate(Sender: TObject);
begin
iLength := 255;
setLength(sWinDir,iLength);
iLength := GetWindowsDirectory(PChar(sWinDir),iLength);
setLength(sWinDir,iLength);
 {someone might have windows installed off the root directory}
if sWinDir[length(sWinDir)] <> '\' then
   appendstr(sWindir,'\');

inidir := sWindir + ininame;
dlldir := sWindir + DLLName;

if not LoadDLLandCreateSafe then
   begin
       showmessage('Terminating Application');
       application.terminate;
   end;

jbitmapoff := tbitmap.create;
imagelist1.getbitmap(0,jbitmapoff );
turnbuttonsoff;
close1.ShortCut := ShortCut(Word('X'),[ssAlt]);
opt := 0;
HistStr := TStringlist.create;  //for file history
Locofsep := pred(Files.count);
if fileexists(inidir) then
   begin
      HistStr.loadfromfile(inidir);
      AddHistory;
   end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.RB1Click(Sender: TObject);
begin
with sender as tBitBtn do
   begin
      turnbuttonsoff;
      imagelist1.getbitmap(1,glyph);
      buttonson := true;
      opt := tag;
      listview1.items.clear;
      DoMain;
   end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.turnbuttonsoff;
begin
but1.glyph := jbitmapoff;
but2.glyph := jbitmapoff;
but3.glyph := jbitmapoff;
but4.glyph := jbitmapoff;
buttonson  := false;
opt        := 0;
end;
//-------------------------------------------------------------------------
procedure Tmainform.Refresh1Click(Sender: TObject);
begin
if opt in [1..8] then
   DoMain;
end;
//-------------------------------------------------------------------------
procedure Tmainform.Open1Click(Sender: TObject);
begin
selbtnclick(sender);
end;
//-------------------------------------------------------------------------
procedure Tmainform.About1Click(Sender: TObject);
begin
Application.CreateForm(Taboutfrm,aboutfrm);
aboutfrm.showmodal;
end;
//-------------------------------------------------------------------------
procedure tmainform.additems(Fnum,Fusr: String);
var NlistItem : TListItem;
begin
NlistItem := Listview1.items.add;
with NlistItem do
   begin
       caption := Fnum;
       subitems.add(Fusr);
   end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 if MessageDlg('Terminate this application?',
    mtConfirmation, [mbYes, mbNo], 0) = mrNo then
    action := CaNone
 else
    begin
        jbitmapoff.free;
        HistStr.savetofile(inidir);
        HistStr.free;
        DestroySafeArray;
        UnloadDll;
    end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.AddHistory;
var
  MItem   : TMenuItem;
  I, Totitems : Integer;
begin
  Totitems := Files.count - 1;
  while Totitems <> Locofsep do
      begin
         files.delete(Totitems);
         dec(Totitems);
      end;
  if Histstr.count > 0 then
     begin
         MItem := TMenuItem.Create(Self);
         Mitem.Caption := 'C&lear History';
         Mitem.onClick := ClearHistory1Click;
         Files.add(Mitem);
         MItem := TMenuItem.Create(Self);
         Mitem.Caption := '-';
         Files.add(Mitem);
         for I := 0 to HistStr.count - 1 do
             begin
                MItem := TMenuItem.Create(Self);
                with MItem do
                   begin
                     onClick := HistoryItemClick;
                     Caption := '&' + inttostr(I) + ' ' + HistStr[I];
                     Files.Add(MItem);
                   end;
             end;
     end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.HistoryItemClick(Sender: TObject );
Begin
with sender as TmenuItem, HistStr do
    begin
       {skip accelerator char, item number and space }
       gdatabase := copy(caption,4,length(caption) - 3); //set db name.
       edit1.text := gdatabase;               //File name
    end;
end;
//-------------------------------------------------------------------------
procedure Tmainform.ClearHistory1Click(Sender: TObject);
begin
HistStr.clear;
AddHistory;
end;
//-------------------------------------------------------------------------
procedure Tmainform.Close1Click(Sender: TObject);
begin
close;
end;
//-------------------------------------------------------------------------
procedure Tmainform.Edit1Change(Sender: TObject);
begin
if not buttonson then
   exit;
listview1.items.clear;
turnbuttonsoff;
end;
//-------------------------------------------------------------------------
end.


